@@ -23,12 +23,12 @@ from typing import TypeVar
 from babi.buf import Buf
 from babi.buf import Modification
 from babi.color_manager import ColorManager
+from babi.dim import Dim
 from babi.hl.interface import FileHL
 from babi.hl.interface import HLFactory
 from babi.hl.replace import Replace
 from babi.hl.selection import Selection
 from babi.hl.trailing_whitespace import TrailingWhitespace
-from babi.margin import Margin
 from babi.prompt import PromptResult
 from babi.status import Status
 
@@ -239,7 +239,7 @@ class File:
     def ensure_loaded(
             self,
             status: Status,
-            margin: Margin,
+            dim: Dim,
             stdin: str,
     ) -> None:
         if self.buf:
@@ -272,7 +272,7 @@ class File:
 
         self._initialize_highlighters()
 
-        self.go_to_line(self.initial_line, margin)
+        self.go_to_line(self.initial_line, dim)
 
     def _initialize_highlighters(self) -> None:
         file_hls = []
@@ -286,6 +286,7 @@ class File:
             *file_hls,
             self._trailing_whitespace, self._replace_hl, self.selection,
         )
+        self.buf.clear_callbacks()
         for file_hl in self._file_hls:
             file_hl.register_callbacks(self.buf)
 
@@ -294,10 +295,10 @@ class File:
             hl_factories: tuple[HLFactory, ...],
             color_manager: ColorManager,
     ) -> None:
+        self._trailing_whitespace = TrailingWhitespace(color_manager)
         self._hl_factories = hl_factories
         # only re-initialize the highlighters if we've loaded once
         if self._file_hls:
-            self._trailing_whitespace = TrailingWhitespace(color_manager)
             self._initialize_highlighters()
 
     def __repr__(self) -> str:
@@ -314,43 +315,43 @@ class File:
     # movement
 
     @action
-    def up(self, margin: Margin) -> None:
-        self.buf.up(margin)
+    def up(self, dim: Dim) -> None:
+        self.buf.up(dim)
 
     @action
-    def down(self, margin: Margin) -> None:
-        self.buf.down(margin)
+    def down(self, dim: Dim) -> None:
+        self.buf.down(dim)
 
     @action
-    def right(self, margin: Margin) -> None:
-        self.buf.right(margin)
+    def right(self, dim: Dim) -> None:
+        self.buf.right(dim)
 
     @action
-    def left(self, margin: Margin) -> None:
-        self.buf.left(margin)
+    def left(self, dim: Dim) -> None:
+        self.buf.left(dim)
 
     @action
-    def home(self, margin: Margin) -> None:
+    def home(self, dim: Dim) -> None:
         self.buf.x = 0
 
     @action
-    def end(self, margin: Margin) -> None:
+    def end(self, dim: Dim) -> None:
         self.buf.x = len(self.buf[self.buf.y])
 
     @action
-    def ctrl_up(self, margin: Margin) -> None:
-        self.buf.file_up(margin)
+    def ctrl_up(self, dim: Dim) -> None:
+        self.buf.file_up(dim)
 
     @action
-    def ctrl_down(self, margin: Margin) -> None:
-        self.buf.file_down(margin)
+    def ctrl_down(self, dim: Dim) -> None:
+        self.buf.file_down(dim)
 
     @action
-    def ctrl_right(self, margin: Margin) -> None:
+    def ctrl_right(self, dim: Dim) -> None:
         line = self.buf[self.buf.y]
         # if we're at the second to last character, jump to end of line
         if self.buf.x == len(line) - 1:
-            self.buf.right(margin)
+            self.buf.right(dim)
         # if we're at the end of the line, jump forward to the next non-ws
         elif self.buf.x == len(line):
             while (
@@ -359,45 +360,45 @@ class File:
                         self.buf[self.buf.y][self.buf.x].isspace()
                     )
             ):
-                self.buf.right(margin)
+                self.buf.right(dim)
         # if we're inside the line, jump to next position that's not our type
         else:
-            self.buf.right(margin)
+            self.buf.right(dim)
             tp = line[self.buf.x].isalnum()
             while self.buf.x < len(line) and tp == line[self.buf.x].isalnum():
-                self.buf.right(margin)
+                self.buf.right(dim)
 
     @action
-    def ctrl_left(self, margin: Margin) -> None:
+    def ctrl_left(self, dim: Dim) -> None:
         line = self.buf[self.buf.y]
         # if we're at position 1 and it's not a space, go to the beginning
         if self.buf.x == 1 and not line[:self.buf.x].isspace():
-            self.buf.left(margin)
+            self.buf.left(dim)
         # if we're at the beginning or it's all space up to here jump to the
         # end of the previous non-space line
         elif self.buf.x == 0 or line[:self.buf.x].isspace():
             self.buf.x = 0
             while self.buf.y > 0 and self.buf.x == 0:
-                self.buf.left(margin)
+                self.buf.left(dim)
         else:
-            self.buf.left(margin)
+            self.buf.left(dim)
             tp = line[self.buf.x - 1].isalnum()
             while self.buf.x > 0 and tp == line[self.buf.x - 1].isalnum():
-                self.buf.left(margin)
+                self.buf.left(dim)
 
     @action
-    def ctrl_home(self, margin: Margin) -> None:
+    def ctrl_home(self, dim: Dim) -> None:
         self.buf.x = 0
         self.buf.y = self.buf.file_y = 0
 
     @action
-    def ctrl_end(self, margin: Margin) -> None:
+    def ctrl_end(self, dim: Dim) -> None:
         self.buf.x = 0
         self.buf.y = len(self.buf) - 1
-        self.buf.scroll_screen_if_needed(margin)
+        self.buf.scroll_screen_if_needed(dim)
 
     @action
-    def go_to_line(self, lineno: int, margin: Margin) -> None:
+    def go_to_line(self, lineno: int, dim: Dim) -> None:
         self.buf.x = 0
         if lineno == 0:
             self.buf.y = 0
@@ -407,14 +408,14 @@ class File:
             self.buf.y = max(0, lineno + len(self.buf))
         else:
             self.buf.y = lineno - 1
-        self.buf.scroll_screen_if_needed(margin)
+        self.buf.scroll_screen_if_needed(dim)
 
     @action
     def search(
             self,
             reg: Pattern[str],
             status: Status,
-            margin: Margin,
+            dim: Dim,
     ) -> None:
         search = _SearchIter(self, reg, offset=1)
         try:
@@ -429,7 +430,7 @@ class File:
                     status.update('search wrapped')
                 self.buf.y = line_y
                 self.buf.x = match.start()
-                self.buf.scroll_screen_if_needed(margin)
+                self.buf.scroll_screen_if_needed(dim)
 
     @clear_selection
     def replace(
@@ -447,7 +448,7 @@ class File:
             end = match.end()
             self.buf.y = line_y
             self.buf.x = match.start()
-            self.buf.scroll_screen_if_needed(screen.margin)
+            self.buf.scroll_screen_if_needed(screen.layout.file)
             if res != 'a':  # make `a` replace the rest of them
                 with self._replace_hl.region(self.buf.y, self.buf.x, end):
                     screen.draw()
@@ -488,33 +489,36 @@ class File:
             occurrences = 'occurrence' if count == 1 else 'occurrences'
             screen.status.update(f'replaced {count} {occurrences}')
 
+    def _page_size(self, dim: Dim) -> int:
+        return max(dim.height - 2, 1)
+
     @action
-    def page_up(self, margin: Margin) -> None:
-        if self.buf.y < margin.body_lines:
+    def page_up(self, dim: Dim) -> None:
+        if self.buf.y < dim.height:
             self.buf.y = self.buf.file_y = 0
         else:
-            pos = max(self.buf.file_y - margin.page_size, 0)
+            pos = max(self.buf.file_y - self._page_size(dim), 0)
             self.buf.y = self.buf.file_y = pos
         self.buf.x = 0
 
     @action
-    def page_down(self, margin: Margin) -> None:
-        if self.buf.file_y + margin.body_lines >= len(self.buf):
+    def page_down(self, dim: Dim) -> None:
+        if self.buf.file_y + dim.height >= len(self.buf):
             self.buf.y = len(self.buf) - 1
         else:
-            pos = self.buf.file_y + margin.page_size
+            pos = self.buf.file_y + self._page_size(dim)
             self.buf.y = self.buf.file_y = pos
         self.buf.x = 0
 
     @action
-    def alt_up(self, margin: Margin) -> None:
+    def alt_up(self, dim: Dim) -> None:
         if self.buf.y > 0:
             offset = 1
             while self.buf[self.buf.y - offset] == '':
                 offset += 1
             if offset >= 2:
                 self.buf.y -= offset
-                self.buf.scroll_screen_if_needed(margin)
+                self.buf.scroll_screen_if_needed(dim)
                 self.buf.x = 0
                 return
             while (
@@ -523,18 +527,18 @@ class File:
             ):
                 offset += 1
             self.buf.y -= offset
-            self.buf.scroll_screen_if_needed(margin)
+            self.buf.scroll_screen_if_needed(dim)
             self.buf.x = 0
 
     @action
-    def alt_down(self, margin: Margin) -> None:
+    def alt_down(self, dim: Dim) -> None:
         if self.buf.y < len(self.buf) - 1:
             offset = 0
             while self.buf[self.buf.y + offset] != '':
                 offset += 1
             if offset > 1:
                 self.buf.y += offset - 1
-                self.buf.scroll_screen_if_needed(margin)
+                self.buf.scroll_screen_if_needed(dim)
                 self.buf.x = 0
                 return
             while self.buf[self.buf.y + offset] == '':
@@ -542,14 +546,14 @@ class File:
                     break
                 offset += 1
             self.buf.y += offset
-            self.buf.scroll_screen_if_needed(margin)
+            self.buf.scroll_screen_if_needed(dim)
             self.buf.x = 0
 
     # editing
 
     @edit_action('backspace text', final=False)
     @clear_selection
-    def backspace(self, margin: Margin) -> None:
+    def backspace(self, dim: Dim) -> None:
         # backspace at the beginning of the file does nothing
         if self.buf.y == 0 and self.buf.x == 0:
             pass
@@ -559,21 +563,21 @@ class File:
                 # still allow backspace if there are 2+ blank lines
                 self.buf[self.buf.y - 1] != ''
         ):
-            self.buf.left(margin)
+            self.buf.left(dim)
         # at the beginning of the line, we join the current line and
         # the previous line
         elif self.buf.x == 0:
             y, victim = self.buf.y, self.buf.pop(self.buf.y)
-            self.buf.left(margin)
+            self.buf.left(dim)
             self.buf[y - 1] += victim
         else:
             s = self.buf[self.buf.y]
             self.buf[self.buf.y] = s[:self.buf.x - 1] + s[self.buf.x:]
-            self.buf.left(margin)
+            self.buf.left(dim)
 
     @edit_action('delete text', final=False)
     @clear_selection
-    def delete(self, margin: Margin) -> None:
+    def delete(self, dim: Dim) -> None:
         if (
             # noop at end of the file
             self.buf.y == len(self.buf) - 1 or
@@ -594,15 +598,15 @@ class File:
 
     @edit_action('line break', final=False)
     @clear_selection
-    def enter(self, margin: Margin) -> None:
+    def enter(self, dim: Dim) -> None:
         s = self.buf[self.buf.y]
         self.buf[self.buf.y] = s[:self.buf.x]
         self.buf.insert(self.buf.y + 1, s[self.buf.x:])
-        self.buf.down(margin)
+        self.buf.down(dim)
         self.buf.x = 0
 
     @edit_action('indent selection', final=True)
-    def _indent_selection(self, margin: Margin) -> None:
+    def _indent_selection(self, dim: Dim) -> None:
         assert self.selection.start is not None
         sel_y, sel_x = self.selection.start
         (s_y, _), (e_y, _) = self.selection.get()
@@ -618,7 +622,7 @@ class File:
         self.selection.set(sel_y, sel_x, self.buf.y, self.buf.x)
 
     @edit_action('insert tab', final=False)
-    def _tab(self, margin: Margin) -> None:
+    def _tab(self, dim: Dim) -> None:
         tab_string = self.buf.tab_string
         if tab_string == '\t':
             n = 1
@@ -632,11 +636,11 @@ class File:
         self.buf.x += n
         self.buf.restore_eof_invariant()
 
-    def tab(self, margin: Margin) -> None:
+    def tab(self, dim: Dim) -> None:
         if self.selection.start is not None:
-            self._indent_selection(margin)
+            self._indent_selection(dim)
         else:
-            self._tab(margin)
+            self._tab(dim)
 
     def _dedent_line(self, s: str) -> int:
         bound = min(len(s), len(self.buf.tab_string))
@@ -646,7 +650,7 @@ class File:
         return i
 
     @edit_action('dedent selection', final=True)
-    def _dedent_selection(self, margin: Margin) -> None:
+    def _dedent_selection(self, dim: Dim) -> None:
         assert self.selection.start is not None
         sel_y, sel_x = self.selection.start
         (s_y, _), (e_y, _) = self.selection.get()
@@ -661,21 +665,21 @@ class File:
         self.selection.set(sel_y, sel_x, self.buf.y, self.buf.x)
 
     @edit_action('dedent', final=True)
-    def _dedent(self, margin: Margin) -> None:
+    def _dedent(self, dim: Dim) -> None:
         n = self._dedent_line(self.buf[self.buf.y])
         if n:
             self.buf[self.buf.y] = self.buf[self.buf.y][n:]
             self.buf.x = max(self.buf.x - n, 0)
 
-    def shift_tab(self, margin: Margin) -> None:
+    def shift_tab(self, dim: Dim) -> None:
         if self.selection.start is not None:
-            self._dedent_selection(margin)
+            self._dedent_selection(dim)
         else:
-            self._dedent(margin)
+            self._dedent(dim)
 
     @edit_action('cut selection', final=True)
     @clear_selection
-    def cut_selection(self, margin: Margin) -> tuple[str, ...]:
+    def cut_selection(self, dim: Dim) -> tuple[str, ...]:
         ret = []
         (s_y, s_x), (e_y, e_x) = self.selection.get()
         if s_y == e_y:
@@ -692,7 +696,7 @@ class File:
                 self.buf.pop(s_y + 1)
         self.buf.y = s_y
         self.buf.x = s_x
-        self.buf.scroll_screen_if_needed(margin)
+        self.buf.scroll_screen_if_needed(dim)
         return tuple(ret)
 
     def cut(self, cut_buffer: tuple[str, ...]) -> tuple[str, ...]:
@@ -708,33 +712,33 @@ class File:
                 self.buf.x = 0
                 return cut_buffer + (victim,)
 
-    def _uncut(self, cut_buffer: tuple[str, ...], margin: Margin) -> None:
+    def _uncut(self, cut_buffer: tuple[str, ...], dim: Dim) -> None:
         for cut_line in cut_buffer:
             line = self.buf[self.buf.y]
             before, after = line[:self.buf.x], line[self.buf.x:]
             self.buf[self.buf.y] = before + cut_line
             self.buf.insert(self.buf.y + 1, after)
-            self.buf.down(margin)
+            self.buf.down(dim)
             self.buf.x = 0
 
     @edit_action('uncut', final=True)
     @clear_selection
-    def uncut(self, cut_buffer: tuple[str, ...], margin: Margin) -> None:
-        self._uncut(cut_buffer, margin)
+    def uncut(self, cut_buffer: tuple[str, ...], dim: Dim) -> None:
+        self._uncut(cut_buffer, dim)
 
     @edit_action('uncut selection', final=True)
     @clear_selection
     def uncut_selection(
             self,
-            cut_buffer: tuple[str, ...], margin: Margin,
+            cut_buffer: tuple[str, ...], dim: Dim,
     ) -> None:
-        self._uncut(cut_buffer, margin)
-        self.buf.up(margin)
+        self._uncut(cut_buffer, dim)
+        self.buf.up(dim)
         self.buf.x = len(self.buf[self.buf.y])
         self.buf[self.buf.y] += self.buf.pop(self.buf.y + 1)
         self.buf.restore_eof_invariant()
 
-    def _sort(self, margin: Margin, s_y: int, e_y: int, reverse: bool) -> None:
+    def _sort(self, dim: Dim, s_y: int, e_y: int, reverse: bool) -> None:
         # self.buf intentionally does not support slicing so we use islice
         lines = sorted(itertools.islice(self.buf, s_y, e_y), reverse=reverse)
         for i, line in zip(range(s_y, e_y), lines):
@@ -742,7 +746,7 @@ class File:
 
         self.buf.y = s_y
         self.buf.x = 0
-        self.buf.scroll_screen_if_needed(margin)
+        self.buf.scroll_screen_if_needed(dim)
 
     def _selection_lines(self) -> tuple[int, int]:
         (s_y, _), (e_y, _) = self.selection.get()
@@ -752,14 +756,14 @@ class File:
         return s_y, e_y
 
     @edit_action('sort', final=True)
-    def sort(self, margin: Margin, reverse: bool = False) -> None:
-        self._sort(margin, 0, len(self.buf) - 1, reverse=reverse)
+    def sort(self, dim: Dim, reverse: bool = False) -> None:
+        self._sort(dim, 0, len(self.buf) - 1, reverse=reverse)
 
     @edit_action('sort selection', final=True)
     @clear_selection
-    def sort_selection(self, margin: Margin, reverse: bool = False) -> None:
+    def sort_selection(self, dim: Dim, reverse: bool = False) -> None:
         s_y, e_y = self._selection_lines()
-        self._sort(margin, s_y, e_y, reverse=reverse)
+        self._sort(dim, s_y, e_y, reverse=reverse)
 
     def _is_commented(self, lineno: int, prefix: str) -> bool:
         return self.buf[lineno].lstrip().startswith(prefix)
@@ -817,7 +821,7 @@ class File:
             else:
                 self._comment_add(lineno, prefix, minimum_indent)
 
-    def reload(self, status: Status, margin: Margin) -> None:
+    def reload(self, status: Status, dim: Dim) -> None:
         assert self.filename is not None
         try:
             lines, nl, mixed, sha256 = _load_file(self.filename)
@@ -830,7 +834,7 @@ class File:
         with self.edit_action_context('reload', final=True):
             self.buf.replace_lines(lines)
 
-        self.buf.fixup_position(margin)
+        self.buf.fixup_position(dim)
 
         if mixed:
             status.update(
@@ -888,7 +892,7 @@ class File:
 
     @edit_action('text', final=False)
     @clear_selection
-    def c(self, wch: str, margin: Margin) -> None:
+    def c(self, wch: str, dim: Dim) -> None:
         s = self.buf[self.buf.y]
         self.buf[self.buf.y] = s[:self.buf.x] + wch + s[self.buf.x:]
         self.buf.x += len(wch)
@@ -959,24 +963,24 @@ class File:
     def move_cursor(
             self,
             stdscr: curses._CursesWindow,
-            margin: Margin,
+            dim: Dim,
     ) -> None:
-        stdscr.move(*self.buf.cursor_position(margin))
+        stdscr.move(*self.buf.cursor_position(dim))
 
-    def draw(self, stdscr: curses._CursesWindow, margin: Margin) -> None:
-        to_display = min(self.buf.displayable_count, margin.body_lines)
+    def draw(self, stdscr: curses._CursesWindow, dim: Dim) -> None:
+        to_display = min(self.buf.displayable_count, dim.height)
 
         for file_hl in self._file_hls:
             # XXX: this will go away?
             file_hl.highlight_until(self.buf, self.buf.file_y + to_display)
 
         for i in range(to_display):
-            draw_y = i + margin.header
+            draw_y = i + dim.y
             l_y = self.buf.file_y + i
-            stdscr.insstr(draw_y, 0, self.buf.rendered_line(l_y, margin))
+            stdscr.insstr(draw_y, 0, self.buf.rendered_line(l_y, dim))
 
-            l_x = self.buf.line_x(margin) if l_y == self.buf.y else 0
-            l_x_max = l_x + margin.cols
+            l_x = self.buf.line_x(dim) if l_y == self.buf.y else 0
+            l_x_max = l_x + dim.width
             for file_hl in self._file_hls:
                 for region in file_hl.regions[l_y]:
                     l_positions = self.buf.line_positions(l_y)
@@ -1003,14 +1007,14 @@ class File:
 
                     if r_end >= l_x_max and l_x_max < l_positions[-1]:
                         if file_hl.include_edge:
-                            h_e_x = margin.cols
+                            h_e_x = dim.width
                         else:
-                            h_e_x = margin.cols - 1
+                            h_e_x = dim.width - 1
                     else:
                         h_e_x = r_end - l_x
 
                     stdscr.chgat(draw_y, h_s_x, h_e_x - h_s_x, region.attr)
 
-        for i in range(to_display, margin.body_lines):
-            stdscr.move(i + margin.header, 0)
+        for i in range(to_display, dim.height):
+            stdscr.move(i + dim.y, 0)
             stdscr.clrtoeol()
